/*******************************************************************************
 * Copyright (c) 2009 TopQuadrant, Inc.
 * All rights reserved. 
 *******************************************************************************/
package org.topbraid.spin.internal;

import org.topbraid.spin.model.Construct;
import org.topbraid.spin.model.ElementList;
import org.topbraid.spin.model.Query;
import org.topbraid.spin.model.Triple;
import org.topbraid.spin.model.TriplePath;
import org.topbraid.spin.model.TriplePattern;
import org.topbraid.spin.model.TripleTemplate;
import org.topbraid.spin.model.Variable;
import org.topbraid.spin.model.visitor.AbstractElementVisitor;
import org.topbraid.spin.model.visitor.AbstractExpressionVisitor;
import org.topbraid.spin.model.visitor.ElementVisitor;
import org.topbraid.spin.model.visitor.ElementWalker;
import org.topbraid.spin.model.visitor.ExpressionVisitor;

import com.hp.hpl.jena.rdf.model.Resource;


/**
 * Can be used to check whether a given SPIN Query contains a certain
 * variable, either in the WHERE clause or the CONSTRUCT template(s).
 * 
 * @author Holger Knublauch
 */
public class ContainsVarChecker {
	
	private boolean result;
	
	private Resource var;
	

	public boolean contains(Query query, Resource var_) {
		
		this.var = var_;

		if(query instanceof Construct) {
			// Check head of Construct
			for(TripleTemplate template : ((Construct)query).getTemplates()) {
				if(containsVar(template)) {
					return true;
				}
			}
		}
		
		// Check WHERE clause
		ElementVisitor el = new AbstractElementVisitor() {

			@Override
			public void visit(TriplePath triplePath) {
				if(var.equals(triplePath.getObject()) ||
					var.equals(triplePath.getSubject())) {
					result = true;
				}
			}

			@Override
			public void visit(TriplePattern triplePattern) {
				if(containsVar(triplePattern)) {
					result = true;
				}
			}
		};
		ExpressionVisitor ex = new AbstractExpressionVisitor() {

			@Override
			public void visit(Variable variable) {
				if(var.equals(variable)) {
					result = true;
				}
			}
		};
		ElementWalker walker = new ElementWalker(el, ex);
		ElementList where = query.getWhere();
		if(where != null) {
			walker.visit(where);
		}
		
		return result;
	}


	private boolean containsVar(Triple triple) {
		return var.equals(triple.getObject()) ||
			var.equals(triple.getPredicate()) ||
			var.equals(triple.getSubject());
	}
}
