/*******************************************************************************
 * Copyright (c) 2009 TopQuadrant, Inc.
 * All rights reserved. 
 *******************************************************************************/
package org.topbraid.spin.model.update.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.topbraid.spin.model.TripleTemplate;
import org.topbraid.spin.model.impl.AbstractSPINResourceImpl;
import org.topbraid.spin.model.print.PrintContext;
import org.topbraid.spin.model.update.Update;
import org.topbraid.spin.vocabulary.SP;

import com.hp.hpl.jena.enhanced.EnhGraph;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Statement;

public abstract class UpdateImpl extends AbstractSPINResourceImpl implements Update {

	public UpdateImpl(Node node, EnhGraph graph) {
		super(node, graph);
	}

	
	private List<TripleTemplate> getTemplates(Property predicate) {
		List<TripleTemplate> results = new LinkedList<TripleTemplate>();
		for(RDFNode next : getList(predicate)) {
			if(next != null && next.isResource()) {
				results.add((TripleTemplate) next.as(TripleTemplate.class));
			}
		}
		return results;
	}


	protected void printGraphIRIs(PrintContext p, String keyword) {
		List<String> graphIRIs = new ArrayList<String>();
		{
			Iterator<Statement> it = listProperties(SP.graphIRI);
			while(it.hasNext()) {
				Statement s = it.next();
				if(s.getObject().isURIResource()) {
					graphIRIs.add(s.getResource().getURI());
				}
			}
			Collections.sort(graphIRIs);
		}
		for(String graphIRI : graphIRIs) {
			p.print(" ");
			if(keyword != null) {
				p.printKeyword(keyword);
				p.print(" ");
			}
			p.printURIResource(getModel().getResource(graphIRI));
		}
	}
	
	
	protected void printTemplates(PrintContext p, Property predicate, String keyword, boolean force) {
		List<TripleTemplate> templates = getTemplates(predicate);
		if(!templates.isEmpty() || force) {
			if(keyword != null) {
				p.printKeyword(keyword);
				p.print(" ");
			}
			p.print(" {");
			p.println();
			for(TripleTemplate template : templates) {
				p.printIndentation(1);
				template.print(p);
				p.print(" .");
				p.println();
			}
			p.print("}\n");
		}
	}


	protected void printWhere(PrintContext p) {
		if(hasProperty(SP.where)) {
			p.printKeyword("WHERE");
			printNestedElementList(p, SP.where);
		}
	}
}
