package org.topbraid.spin.spr;

import java.util.LinkedList;
import java.util.List;

import org.topbraid.spin.system.ARQFactory;
import org.topbraid.spin.util.JenaDatatypes;
import org.topbraid.spin.vocabulary.SPR;

import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QuerySolutionMap;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.Literal;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;


/**
 * Static utilities on SPR tables.
 * 
 * @author Holger Knublauch
 */
public class SPRResultSets {
	
	public static RDFNode getCell(Resource table, int row, int col) {
		Model model = table.getModel();
		String queryString =
			"SELECT ?result\n" +
			"WHERE {\n" +
			"    LET (?result := <" + SPR.cell.getURI() + ">(?table, ?row, ?col)) .\n" +
			"}";
		Query query = ARQFactory.get().createQuery(model, queryString);
		QueryExecution qexec = ARQFactory.get().createQueryExecution(query, model);
		QuerySolutionMap bindings = new QuerySolutionMap();
		bindings.add("table", table);
		bindings.add("row", JenaDatatypes.createInteger(row));
		bindings.add("col", JenaDatatypes.createInteger(col));
		qexec.setInitialBinding(bindings);
		ResultSet rs = qexec.execSelect();
		if(rs.hasNext()) {
			RDFNode result = rs.next().get("result");
			qexec.close();
			return result;
		}
		else {
			return null;
		}
	}

	
	public static int getColCount(Resource table) {
		return getIntFromFunction(table, SPR.colCount);
	}
	
	
	public static String getColName(Resource table, int col) {
		Model model = table.getModel();
		String queryString =
			"SELECT ?result\n" +
			"WHERE {\n" +
			"    LET (?result := <" + SPR.colName.getURI() + ">(?table, ?col)) .\n" +
			"}";
		Query query = ARQFactory.get().createQuery(model, queryString);
		QueryExecution qexec = ARQFactory.get().createQueryExecution(query, model);
		QuerySolutionMap bindings = new QuerySolutionMap();
		bindings.add("table", table);
		bindings.add("col", JenaDatatypes.createInteger(col));
		qexec.setInitialBinding(bindings);
		ResultSet rs = qexec.execSelect();
		if(rs.hasNext()) {
			RDFNode result = rs.next().get("result");
			qexec.close();
			if(result.isLiteral()) {
				return ((Literal)result).getString();
			}
		}
		return null;
	}
	

	public static List<String> getColNames(Resource table) {
		List<String> results = new LinkedList<String>();
		int colCount = getColCount(table);
		for(int i = 0; i < colCount; i++) {
			results.add(getColName(table, i));
		}
		return results;
	}
	
	
	private static int getIntFromFunction(Resource table, Resource function) {
		Model model = table.getModel();
		String queryString =
			"SELECT ?result\n" +
			"WHERE {\n" +
			"    LET (?result := <" + function.getURI() + ">(?table)) .\n" +
			"}";
		Query query = ARQFactory.get().createQuery(model, queryString);
		QueryExecution qexec = ARQFactory.get().createQueryExecution(query, model);
		QuerySolutionMap bindings = new QuerySolutionMap();
		bindings.add("table", table);
		qexec.setInitialBinding(bindings);
		ResultSet rs = qexec.execSelect();
		if(rs.hasNext()) {
			RDFNode result = rs.next().get("result");
			qexec.close();
			if(result.isLiteral()) {
				return ((Literal)result).getInt();
			}
		}
		return 0;
	}

	
	public static int getRowCount(Resource table) {
		return getIntFromFunction(table, SPR.rowCount);
	}
}
