/*******************************************************************************
 * Copyright (c) 2009 TopQuadrant, Inc.
 * All rights reserved. 
 *******************************************************************************/
package org.topbraid.spin.util;

import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;


/**
 * Wraps a (pre-compiled) Jena Query with its source SPIN Query and
 * a human-readable string representation. 
 * 
 * Also needed to work around the issue of Query.equals/hashCode: Otherwise
 * multiple distinct template calls will be merged into one in HashMaps.
 * 
 * @author Holger Knublauch
 */
public class QueryWrapper {
	
	private boolean thisUnbound;
	
	private String label;
	
	private Query query;
	
	private Resource source;
	
	private org.topbraid.spin.model.Query spinQuery;
	
	private Statement statement;
	
	private String text;
	
	
	public QueryWrapper(Query query, Resource source, String text, org.topbraid.spin.model.Query spinQuery, String label, Statement statement, boolean thisUnbound) {
		this.label = label;
		this.query = query;
		this.statement = statement;
		this.source = source;
		this.text = text;
		this.spinQuery = spinQuery;
		this.thisUnbound = thisUnbound;
	}
	
	
	public String getLabel() {
		return label;
	}
	
	
	public Query getQuery() {
		return query;
	}
	
	
	public org.topbraid.spin.model.Query getSPINQuery() {
		return spinQuery;
	}
	
	
	public Statement getStatement() {
		return statement;
	}
	
	
	/**
	 * Gets the SPIN Query or template call that has created this QueryWrapper. 
	 * @return the source
	 */
	public Resource getSource() {
		return source;
	}
	
	
	public String getText() {
		return text;
	}
	
	
	public boolean isThisUnbound() {
		return thisUnbound;
	}
}
