/*******************************************************************************
 * Copyright (c) 2009 TopQuadrant, Inc.
 * All rights reserved. 
 *******************************************************************************/
package org.topbraid.spin.util;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.topbraid.spin.model.Ask;
import org.topbraid.spin.model.Command;
import org.topbraid.spin.model.Construct;
import org.topbraid.spin.model.SPINFactory;
import org.topbraid.spin.model.Template;
import org.topbraid.spin.model.TemplateCall;
import org.topbraid.spin.model.print.StringPrintContext;
import org.topbraid.spin.system.ARQFactory;
import org.topbraid.spin.system.SPINLabels;
import org.topbraid.spin.vocabulary.SPIN;

import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;


/**
 * Can be used to search for all queries associated with a class, e.g. via spin:rule.
 *
 * @author Holger Knublauch
 */
public class SPINQueryFinder {
	

	public static void add(Map<Resource, List<QueryWrapper>> class2Query, Statement s,
			Model model,
			boolean withClass,
			Map<QueryWrapper, Map<String, RDFNode>> initialTemplateBindings,
			boolean allowAsk) {
		if(s.getObject().isResource()) {
			String spinQueryText = null;
			String label = null;
			org.topbraid.spin.model.Query spinQuery = null;
			Template template = null;
			TemplateCall templateCall = SPINFactory.asTemplateCall(s.getResource());
			if(templateCall != null) {
				template = templateCall.getTemplate();
				if(template != null) {
					Command body = template.getBody();
					if(body instanceof Construct || (allowAsk && body instanceof Ask)) {
						spinQuery = (org.topbraid.spin.model.Query) body;
					}
				}
				spinQueryText = SPINLabels.get().getLabel(templateCall);
				label = spinQueryText;
			}
			else {
				spinQuery = SPINFactory.asQuery(s.getResource());
				if(spinQuery != null) {
					label = spinQuery.getComment();
				}
			}
			
			if(spinQuery != null) {
				StringPrintContext p = new StringPrintContext();
				spinQuery.print(p);
				boolean thisUnbound = spinQuery.hasProperty(SPIN.thisUnbound, JenaDatatypes.TRUE);
				if(spinQueryText == null) {
					spinQueryText = p.getString();
				}
				if(!thisUnbound && withClass && spinQuery instanceof Construct && SPINUtil.containsThis(spinQuery)) {
					SPINUtil.addThisTypeClause(p.getStringBuilder());
				}
				QueryWrapper wrapper = null;
				Query arqQuery = ARQFactory.get().createQuery(model, p.getStringBuilder().toString());
				if(arqQuery.isConstructType() || (allowAsk && arqQuery.isAskType())) {
					Resource type = s.getSubject();
					List<QueryWrapper> list = class2Query.get(type);
					if(list == null) {
						list = new LinkedList<QueryWrapper>(); 
						class2Query.put(type, list);
					}
					Resource source = templateCall != null ? templateCall : spinQuery;
					wrapper = new QueryWrapper(arqQuery, source, spinQueryText, spinQuery, label, s, thisUnbound);
					list.add(wrapper);
				}
				
				if(template != null && wrapper != null) {
					Map<String,RDFNode> bindings = templateCall.getArgumentsMapByVarNames();
					if(!bindings.isEmpty()) {
						initialTemplateBindings.put(wrapper, bindings);
					}
				}
			}
		}
	}

	
	/**
	 * Gets a Map of QueryWrappers with their associated classes. 
	 * @param model  the Model to operate on
	 * @param queryModel  the Model to query on (might be different)
	 * @param predicate  the predicate such as <code>spin:rule</code>
	 * @param withClass  true to also include a SPARQL clause to bind ?this
	 *                   (something along the lines of ?this a ?THIS_CLASS) 
	 * @param initialTemplateBindings  will contain the initial bindings if
	 *                                 QueryWrappers wrap SPIN template calls
	 * @param allowAsk  also return ASK queries
	 * @return the result Map, possibly empty but not null
	 */
	public static Map<Resource, List<QueryWrapper>> getClass2QueryMap(Model model, Model queryModel, Property predicate, boolean withClass, Map<QueryWrapper,Map<String,RDFNode>> initialTemplateBindings, boolean allowAsk) {
		predicate = model.getProperty(predicate.getURI());
		Map<Resource,List<QueryWrapper>> class2Query = new HashMap<Resource,List<QueryWrapper>>();
		List<Statement> ss = JenaUtil.getStatementsList(JenaUtil.listAllProperties(null, predicate));
		for(Statement s : ss) {
			add(class2Query, s, model, withClass, initialTemplateBindings, allowAsk);
		}
		return class2Query;
	}

}
